/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "pikaimage.h"
#include "pikaimage-guides.h"
#include "pikaguide.h"
#include "pikaguideundo.h"


static void   pika_guide_undo_constructed  (GObject            *object);

static void   pika_guide_undo_pop          (PikaUndo            *undo,
                                            PikaUndoMode         undo_mode,
                                            PikaUndoAccumulator *accum);


G_DEFINE_TYPE (PikaGuideUndo, pika_guide_undo,
               PIKA_TYPE_AUX_ITEM_UNDO)

#define parent_class pika_guide_undo_parent_class


static void
pika_guide_undo_class_init (PikaGuideUndoClass *klass)
{
  GObjectClass  *object_class = G_OBJECT_CLASS (klass);
  PikaUndoClass *undo_class   = PIKA_UNDO_CLASS (klass);

  object_class->constructed = pika_guide_undo_constructed;

  undo_class->pop           = pika_guide_undo_pop;
}

static void
pika_guide_undo_init (PikaGuideUndo *undo)
{
}

static void
pika_guide_undo_constructed (GObject *object)
{
  PikaGuideUndo *guide_undo = PIKA_GUIDE_UNDO (object);
  PikaGuide     *guide;

  G_OBJECT_CLASS (parent_class)->constructed (object);

  guide = PIKA_GUIDE (PIKA_AUX_ITEM_UNDO (object)->aux_item);

  pika_assert (PIKA_IS_GUIDE (guide));

  guide_undo->orientation = pika_guide_get_orientation (guide);
  guide_undo->position    = pika_guide_get_position (guide);
}

static void
pika_guide_undo_pop (PikaUndo              *undo,
                     PikaUndoMode           undo_mode,
                     PikaUndoAccumulator   *accum)
{
  PikaGuideUndo       *guide_undo = PIKA_GUIDE_UNDO (undo);
  PikaGuide           *guide;
  PikaOrientationType  orientation;
  gint                 position;
  gboolean             moved = FALSE;

  PIKA_UNDO_CLASS (parent_class)->pop (undo, undo_mode, accum);

  guide = PIKA_GUIDE (PIKA_AUX_ITEM_UNDO (undo)->aux_item);

  orientation = pika_guide_get_orientation (guide);
  position    = pika_guide_get_position (guide);

  if (position == PIKA_GUIDE_POSITION_UNDEFINED)
    {
      pika_image_add_guide (undo->image, guide, guide_undo->position);
    }
  else if (guide_undo->position == PIKA_GUIDE_POSITION_UNDEFINED)
    {
      pika_image_remove_guide (undo->image, guide, FALSE);
    }
  else
    {
      pika_guide_set_position (guide, guide_undo->position);

      moved = TRUE;
    }

  pika_guide_set_orientation (guide, guide_undo->orientation);

  if (moved || guide_undo->orientation != orientation)
    pika_image_guide_moved (undo->image, guide);

  guide_undo->position    = position;
  guide_undo->orientation = orientation;
}
